/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.biblioteca;

import jakarta.persistence.Basic;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.Id;
import jakarta.persistence.Index;
import jakarta.persistence.Table;
import jakarta.persistence.Temporal;
import static jakarta.persistence.TemporalType.DATE;
import static jakarta.persistence.TemporalType.TIMESTAMP;
import java.io.Serializable;
import java.util.Date;

@Entity
@Table(
        indexes= {
            @Index(name="FITXA_IDX_TITOL", columnList="titol")
        }
)
public class Fitxa implements Comparable<Fitxa>, Serializable{

    @Id
    @Column(columnDefinition="varchar(10) constraint FITXA_REF_LEN_CK check(length(referencia)=10)")
    private String referencia;          // Obligatòria i de 10 caràcters
//    @Basic (optional=false)
//    @Column(nullable=false, length=60)
    @Column(columnDefinition="varchar(60) not null constraint FITXA_TIT_LEN_CK check(length(titol)>0)")
    private String titol;               // Obligatori no buit
    @Column(name="es_deixa")
    private Boolean esDeixa;            // No obligatori
    @Basic(optional=false)
    @Column(nullable=false, name="moment_creacio", updatable=false)
    @Temporal(TIMESTAMP)    // Per defecte, és TIMESTAMP
    private Date momentCreacio = new Date();  // Inicialitzada amb la data del sistema
    @Column(name="moment_modificacio")
    @Temporal(TIMESTAMP)    // Per defecte, és TIMESTAMP
    private Date momentModificacio;  // Inicialitzada a NULL
    // Només apuntarà una data a partir del moment en que
    // es modifiqui per primera vegada

    protected Fitxa(){
        
    }
    
    public Fitxa(String referencia, String titol) {
        assignReferencia(referencia);
        assignTitol(titol);
    }

    public Fitxa(String referencia, String titol, Boolean esDeixa) {
        assignReferencia(referencia);
        assignTitol(titol);
        assignEsDeixa(esDeixa);
    }

    public Fitxa(String referencia, String titol, Boolean esDeixa, Date momentCreacio, Date momentModificacio) {
        if (momentCreacio == null) {
            throw new FitxaException("Intent de creació de fitxa amb data de creació nul·la", 'C');
        }
        if (momentModificacio != null && momentModificacio.compareTo(momentCreacio) <= 0) {
            throw new FitxaException("Intent de creació de fitxa amb data de modificació anterior a data de creació", 'M');
        }
        assignReferencia(referencia);
        assignTitol(titol);
        assignEsDeixa(esDeixa);
        this.momentCreacio = (Date) momentCreacio.clone();
        this.momentModificacio = (Date) momentModificacio.clone();
    }

    public String getReferencia() {
        return referencia;
    }

    public void setReferencia(String referencia) {
        assignReferencia(referencia);
        setMomentModificacio();
    }

    private void assignReferencia(String referencia) {
        if (referencia == null) {
            throw new FitxaException("La referència és obligatòria.", 'R', referencia);
        }
        if (referencia.length() != 10) {
            throw new FitxaException("La referència ha de tenir 10 caràcters de llargada", 'R', referencia);
        }
        this.referencia = referencia;
    }

    public String getTitol() {
        return titol;
    }

    public void setTitol(String titol) {
        assignTitol(titol);
        setMomentModificacio();
    }

    private void assignTitol(String titol) {
        if (titol == null || titol.length() == 0) {
            throw new FitxaException("Títol obligatori i no buït", 'T', titol);
        }
        this.titol = titol;
    }

    public Boolean getEsDeixa() {
        return esDeixa;
    }

    public void setEsDeixa(Boolean esDeixa) {
        assignEsDeixa(esDeixa);
        setMomentModificacio();
    }

    private void assignEsDeixa(Boolean esDeixa) {
        this.esDeixa = esDeixa;
    }

    public Date getMomentCreacio() {
        return momentCreacio;
    }

    public Date getMomentModificacio() {
        return momentModificacio;
    }

    @Override
    public String toString() {
        return "Fitxa{" + "referencia=" + referencia + ", titol=" + titol + ", esDeixa=" + esDeixa + ", momentCreacio=" + momentCreacio + ", momentModificacio=" + momentModificacio + '}';
    }

    @Override
    /**
     * Dues fitxes són iguals si pertanyen a la mateixa classe i tenen la
     * mateixa referència
     */
    public final boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null) {
            return false;
        }
        if (getClass() != o.getClass()) {
            return false;
        }
        Fitxa f = (Fitxa) o;
        return referencia.equals(f.referencia);
    }

    @Override
    public final int hashCode() {
        return 7 * referencia.hashCode();
    }

    public void visualitzar() {
        System.out.println("\tReferència...........: " + referencia);
        System.out.println("\tTítol................: " + titol);
        System.out.println("\tEs Deixa?............: " + esDeixa);
        System.out.println("\tData Creació.........: " + momentCreacio);
        System.out.println("\tData Last Modificació: " + momentModificacio);
    }

    protected final void setMomentModificacio() {
        if (momentModificacio == null) {
            momentModificacio = new Date();
        } else {
            momentModificacio.setTime(System.currentTimeMillis());
        }
    }

    @Override
    public int compareTo(Fitxa o) {
        return referencia.compareTo(o.referencia);
    }

};
