/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.biblioteca;

import jakarta.persistence.Basic;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.ForeignKey;
import jakarta.persistence.Id;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToOne;
import java.io.Serializable;
import java.util.Date;

/**
 *
 * @author Usuari
 */
@Entity
public class Prestec implements Serializable {

    @Id
    private long numero;    // Obligatori - Immutable
    
    @ManyToOne(optional=false, cascade = CascadeType.PERSIST,
            fetch = FetchType.LAZY)
    @JoinColumn(name="soci", nullable=false,
            foreignKey=@ForeignKey(name="prestec_fk_soci"))
    private Soci soci;  // Obligatori - Immutable
    
    @ManyToOne(optional=false, cascade = CascadeType.PERSIST,
            fetch = FetchType.LAZY)
    @JoinColumn(name="fitxa", nullable=false,
            foreignKey=@ForeignKey(name="prestec_fk_fitxa"))
    private Fitxa fitxa;    // Obligatori - Immutable
    
    @Basic(optional=false)
    @Column(name="moment_prestec", nullable=false)
    private Date momentPrestec;   // Obligatori   (moment temporal) - Immutable
    @Column(name="moment_retorn")
    private Date momentRetorn;    // No obligatori (moment temporal) - Posterior a momentPrestec
    // null = Pendent de retorn

    protected Prestec() {
    }

    public Prestec(long numero, Soci soci, Fitxa fitxa, Date momentPrestec) {
        setNumero(numero);
        setSoci(soci);
        setFitxa(fitxa);
        setMomentPrestec(momentPrestec);
    }

    public long getNumero() {
        return numero;
    }

    private void setNumero(long numero) {
        if (numero <= 0) {
            throw new PrestecException("En un préstec, el número és estrictament positiu");
        }
        this.numero = numero;
    }

    public Soci getSoci() {
        return soci;
    }

    public void setSoci(Soci soci) {
        if (soci == null) {
            throw new PrestecException("En un préstec, el soci és obligatori");
        }
        this.soci = soci;
    }

    public  Fitxa getFitxa() {
        return fitxa;
    }

    private void setFitxa(Fitxa fitxa) {
        if (fitxa == null) {
            throw new PrestecException("En un préstec, la fitxa és obligatòria");
        }
        this.fitxa = fitxa;
    }

    public Date getMomentPrestec() {
        return momentPrestec;
    }

    private void setMomentPrestec(Date momentPrestec) {
        if (momentPrestec == null) {
            throw new PrestecException("En un préstec, la data de préstec és obligatòria");
        }
        this.momentPrestec = momentPrestec;
    }

    public Date getMomentRetorn() {
        return momentRetorn;
    }

    public void setMomentRetorn(Date momentRetorn) {
        if (momentRetorn != null && momentRetorn.compareTo(momentPrestec) <= 0) {
            throw new PrestecException("El retorn d'un préstec ha de ser posterior al moment en què s'ha efectuat el préstec");
        }
        this.momentRetorn = momentRetorn;
    }

    @Override
    public String toString() {
        return "Prestec{" + "numero=" + numero + ", soci=" + soci + ", fitxa=" + fitxa + ", momentPrestec=" + momentPrestec + ", momentRetorn=" + momentRetorn + '}';
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 47 * hash + (int) (this.numero ^ (this.numero >>> 32));
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Prestec other = (Prestec) obj;
        if (this.numero != other.numero) {
            return false;
        }
        return true;
    }

}
