/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.biblioteca;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Objects;
import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.ForeignKey;
import jakarta.persistence.Id;
import jakarta.persistence.IdClass;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.Lob;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;
import jakarta.persistence.Temporal;
import jakarta.persistence.TemporalType;
import jakarta.persistence.UniqueConstraint;

/**
 *
 * @author Usuari
 */
@Entity
@Table(name = "agenda_tallers")
// No cal posar uniqueConstraints per què les claus compostes via vàries @Id generen PK a la BD
@IdClass(AgendaTallersId.class)
public class AgendaTallers implements Serializable {

    @Id
    @Column(length = 4, name = "codi_taller")
    private String codiTaller;

    @ManyToOne(optional = false,
            cascade = {CascadeType.PERSIST},
            fetch = FetchType.LAZY)
    @JoinColumn(name = "codi_taller", insertable = false, updatable = false,
            foreignKey = @ForeignKey(name = "agenda_taller_fk_taller"))
    private Taller taller;              // Obligatori

    @Id
    @Column(name = "moment_inici")
    @Temporal(value = TemporalType.DATE)
    private Calendar momentInici;       // Obligatori
    
    
    @Lob
    private String detalls;             // Optatiu o amb contingut

    protected AgendaTallers() {
    }

    public AgendaTallers(Taller taller, Calendar momentInici) {
        setTaller(taller);
        setMomentInici(momentInici);
    }

    public Taller getTaller() {
        return taller;
    }

    private void setTaller(Taller taller) {
        if (taller == null) {
            throw new RuntimeException("Taller obligatori");
        }
        this.taller = taller;
        this.codiTaller = taller.getCodi();
    }

    public Calendar getMomentInici() {
        return momentInici;
    }

    private void setMomentInici(Calendar momentInici) {
        if (momentInici == null) {
            throw new RuntimeException("Moment d'inici obligatori");
        }
        this.momentInici = momentInici;
    }

    public String getDetalls() {
        return detalls;
    }

    public void setDetalls(String detalls) {
        this.detalls = detalls;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 23 * hash + Objects.hashCode(this.taller);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final AgendaTallers other = (AgendaTallers) obj;
        if (!Objects.equals(this.codiTaller, other.codiTaller)) {
            return false;
        }
        if (!Objects.equals(this.momentInici, other.momentInici)) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "AgendaTallers{" + "taller=" + taller + ", momentInici=" + momentInici + ", detalls=" + detalls + '}';
    }

}
